#include "U8glib.h"
#include <arduino.h>

/**********************************************************************
 DISPLAY
 **********************************************************************
 PIN Konfig
 52 SCK
 50 MISO
 51 MOSI /DIN
 53 SS /CS
 32 DC /A0
 31 RESET
 30 BackLight
 ***********************************************************************/

//U8GLIB_PCD8544 u8g(52, 51, 53, 32, 31);// SCK, MOSI, CS, A0,RESET .... SW SPI
U8GLIB_PCD8544 u8g(53, 32, 31); // CS, A0, RESET ..... HW SPI
#define BACKLIGHT 30
#define FONTHEIGTH 7

/***********************************************************************
 *SERVOs steering, speed
 ***********************************************************************
 * auf folgenden Arduino Pins befinden sich die Servoanschlsse:
 *
 *  5 ... throttle / Steering
 *  6 ... esc / speed
 *  ********************************************************************/

#include <Servo.h>
#define PIN_STEERING_SERVO 5
#define PIN_SPEED_SERVO 2

Servo steeringServo;  // SteeringServo
Servo speedServo;  // speedServo

/*************************************************************************
 * IR-Length Sensors
 *************************************************************************
 * auf folgenden PINs befinden sich die IR Sensoren
 *
 * A0 ... VBATT_MEASUREMENT
 * A1 ... Distance Left
 * A2 ... Distance Right
 * A3 ... Distance Middle
 *
 * Anmerkung: Es kann die interne Referenzspannung verwendet werden
 *************************************************************************/

#define LEFTSENSOR A1
#define MIDDLESENSOR A3
#define RIGTHSENSOR A2
#define VBAT A0

/*************************************************************************/

/*************************************************************************
 * Start Stop Tasten
 *************************************************************************
 * auf folgenden PINs befinden sich die Start und die Stop Taste
 *
 * 13 ... Stop
 * 12 ... Start
 *
 * Anmerkung: die Tasten verwenden negative logik
 *************************************************************************/
#define STARTBUTTON 12 //schwarze Taste
#define STOPBUTTON 13  //rote Taste

/**************************************************************************
 * Global Variablen
 **************************************************************************/
unsigned char runMode = 0; //Variable = 0 ... stop

//The setup function is called once at startup of the sketch
void setup() {
	//attach Servo objects to the corresponding PINs
	steeringServo.attach(PIN_STEERING_SERVO);
	steeringServo.write(90); //Set it to neutral position
	speedServo.attach(PIN_SPEED_SERVO);
	setupESCPWM();


	//Hintergrundbeleuchungs pin konfigurieren
	pinMode(BACKLIGHT, OUTPUT);
}

// The loop function is called in an endless loop
void loop() {

	//First get all data
	int leftDistance = analogRead(LEFTSENSOR);
	int middleDistance = analogRead(MIDDLESENSOR);
	int rightDistance = analogRead(RIGTHSENSOR);
	int vBat = analogRead(VBAT);


	//Now check the Button and run control strategies

	if (digitalRead(STOPBUTTON) == LOW)
		runMode = 0; //stopen des Fahrzeugs
	else {
		if (digitalRead(STARTBUTTON) == LOW)
			runMode = 1; //fahrzeug darf fahren
	}

	if (runMode == 1) {
		//Geschwindigkeit
		speedServo.writeMicroseconds(1833); //lowspeed ahead

		//Lenkung (Steering)
		int lenkung = ((long) (leftDistance - middleDistance)) * 90 / 1024;
		steeringServo.write(lenkung + 90);
	}

	//finally update display

	u8g.setFont(u8g_font_4x6);
	u8g.firstPage();

	do {
		draw(leftDistance, middleDistance, rightDistance, vBat);
	} while (u8g.nextPage());

}

void displaySensorValue(const int value, const char *S, const int xPos,
		const int yPos) {

	u8g.setPrintPos(xPos, yPos);
	u8g.print(S);
	u8g.print(value);

	//calc box
	int boxWidth = ((long) value) * 42 / 1024;

	u8g.drawFrame(84 - boxWidth, yPos - FONTHEIGTH, boxWidth, FONTHEIGTH - 1);
}

void draw(const int leftDistance, const int middleDistance,
		const int rightDistance, const int vBat) {
	//Header
	u8g.setFont(u8g_font_4x6);
	u8g.drawStr(0, 7, "Crazy Car");

	displaySensorValue(leftDistance, "left ", 0, FONTHEIGTH * 3);
	displaySensorValue(middleDistance, "middle ", 0, FONTHEIGTH * 4);
	displaySensorValue(rightDistance, "right ", 0, FONTHEIGTH * 5);
	displaySensorValue(vBat, "vBat ", 0, FONTHEIGTH * 6);
}


void setupESCPWM() {
    const int CONFIG_DELAY = 5000;//Zeit wie lange das Konfig Signal anliegt
	speedServo.writeMicroseconds(500);//setup maximum Reverse
	delay(CONFIG_DELAY);
	speedServo.writeMicroseconds(1000);//setup neutral zone Reverse|Brake
	delay(CONFIG_DELAY);
	speedServo.writeMicroseconds(1500);//setup Neutral zone Brake Forward
	delay(CONFIG_DELAY);
	speedServo.writeMicroseconds(2500);//setup Forward Max
	delay(CONFIG_DELAY);
	speedServo.writeMicroseconds(1480);//set into Brake mode
	delay(CONFIG_DELAY);
}
